from dataclasses import dataclass
from enum import Enum
from typing import Optional

Status = Enum("Status", "SUCCESS REJECTED FILTERED")


@dataclass
class Result():
    status: Status
    message: Optional[str] = None


SUCCESS = Result(Status.SUCCESS)
FILTERED = Result(Status.FILTERED)



import logging
from enum import Enum, unique
from typing import Any, Dict, List, Optional, Tuple

import sqlalchemy as sa
from datayoga_core import utils, write_utils
from datayoga_core.block import Block as DyBlock
from datayoga_core.context import Context
from datayoga_core.result import Result
from sqlalchemy import Table
from sqlalchemy.engine import CursorResult
from sqlalchemy.exc import OperationalError
from sqlalchemy.sql.expression import ColumnCollection

logger = logging.getLogger("dy")


@unique
class DbType(Enum):
    MSSQL = "mssql"
    MYSQL = "mysql"
    PSQL = "postgresql"

    @classmethod
    def has_value(cls, value: str) -> bool:
        return value in cls._value2member_map_


DEFAULT_DRIVERS = {
    DbType.MYSQL.value: "mysql+pymysql",
    DbType.MSSQL.value: "mssql+pymssql",
    DbType.PSQL.value: "postgresql"
}

class Block(DyBlock):

    def init(self, context: Optional[Context] = None):
        logger.debug(f"Initializing {self.get_block_name()}")

        connection = utils.get_connection_details(self.properties.get("connection"), context)

        self.db_type = connection.get("type").lower()
        if not DbType.has_value(self.db_type):
            raise ValueError(f"{self.db_type} is not supported yet")

        self.schema = self.properties.get("schema")
        self.table = self.properties.get("table")
        self.opcode_field = self.properties.get("opcode_field")
        self.load_strategy = self.properties.get("load_strategy")
        self.keys = self.properties.get("keys")
        self.mapping = self.properties.get("mapping")

        engine = sa.create_engine(
            sa.engine.URL.create(
                drivername=connection.get("driver", DEFAULT_DRIVERS.get(self.db_type)),
                host=connection.get("host"),
                port=connection.get("port"),
                username=connection.get("user"),
                password=connection.get("password"),
                database=connection.get("database")),
            echo=connection.get("debug", False), connect_args=connection.get("connect_args", {}))
        self.tbl = sa.Table(self.table, sa.MetaData(schema=self.schema), autoload_with=engine)

        logger.debug(f"Connecting to {self.db_type}")
        self.connection = engine.connect()

        if self.opcode_field:
            self.business_key_columns = [column["column"] for column in write_utils.get_column_mapping(self.keys)]
            self.mapping_columns = [column["column"] for column in write_utils.get_column_mapping(self.mapping)]

            self.columns = self.business_key_columns + [x for x in self.mapping_columns
                                                        if x not in self.business_key_columns]

            for column in self.columns:
                if not column in self.tbl.columns:
                    raise ValueError(f"{column} column does not exist in {self.tbl.fullname} table")

            self.delete_stmt = self.tbl.delete().where(
                sa.and_(*[(self.tbl.columns[column] == sa.bindparam(column)) for column in self.business_key_columns]))

            self.upsert_stmt = self.generate_upsert_stmt()

    async def run(self, data: List[Dict[str, Any]]) -> Tuple[List[Dict[str, Any]], List[Result]]:
        logger.debug(f"Running {self.get_block_name()}")

        if self.opcode_field:
            records_to_insert, records_to_update, records_to_delete = write_utils.group_records_by_opcode(
                data, self.opcode_field, self.keys)

            self.execute_upsert(records_to_insert + records_to_update)
            self.execute_delete(records_to_delete)
        else:
            logger.debug(f"Inserting {len(data)} record(s) to {self.table} table")
            self.execute(self.tbl.insert(), data)

        return utils.produce_data_and_results(data)

    def generate_upsert_stmt(self) -> Any:
        """Generates an UPSERT statement based on the DB type"""
        if self.db_type == DbType.PSQL.value:
            from sqlalchemy.dialects.postgresql import insert

            insert_stmt = insert(self.tbl).values({col: "?" for col in self.columns})
            return insert_stmt.on_conflict_do_update(
                index_elements=self.business_key_columns,
                set_={col: getattr(insert_stmt.excluded, col) for col in self.columns})

        if self.db_type == DbType.MYSQL.value:
            from sqlalchemy.dialects.mysql import insert

            insert_stmt = insert(self.tbl).values({col: "?" for col in self.columns})
            return insert_stmt.on_duplicate_key_update(ColumnCollection(
                columns=[(x.name, x) for x in [insert_stmt.inserted[column] for column in self.columns]]))

        if self.db_type == DbType.MSSQL.value:
            return sa.sql.text("""
                    MERGE %s AS target
                    USING (VALUES (%s)) AS source (%s) ON (%s)
                    WHEN NOT MATCHED BY target THEN INSERT (%s) VALUES (%s)
                    WHEN MATCHED THEN UPDATE SET %s;
                    """ % (
                f"[{self.tbl.schema}].[{self.tbl.name}]",
                ", ".join([f"{sa.bindparam(column)}" for column in self.business_key_columns]),
                ", ".join([f"[{column}]" for column in self.business_key_columns]),
                "AND ".join([f"target.[{column}] = source.[{column}]" for column in self.business_key_columns]),
                ", ".join([f"[{column}]" for column in self.columns]),
                ", ".join([f"{sa.bindparam(column)}" for column in self.columns]),
                ", ".join([f"target.[{column}] = {sa.bindparam(column)}" for column in self.mapping_columns])
            ))

    def execute(self, statement: Any, records: List[Dict[str, Any]]) -> CursorResult:
        try:
            return self.connection.execute(statement, records)
        except OperationalError as e:
            raise ConnectionError(e)

    def execute_upsert(self, records: List[Dict[str, Any]]):
        if records:
            logger.debug(f"Upserting {len(records)} record(s) to {self.table} table")
            records_to_upsert = []
            for record in records:
                records_to_upsert.append(write_utils.map_record(record, self.keys, self.mapping))

            if records_to_upsert:
                self.execute(self.upsert_stmt, records_to_upsert)

    def execute_delete(self, records: List[Dict[str, Any]]):
        if records:
            logger.debug(f"Deleting {len(records)} record(s) from {self.table} table")
            records_to_delete = []
            for record in records:
                records_to_delete.append(write_utils.map_record(record, self.keys))

            if records_to_delete:
                self.execute(self.delete_stmt, records_to_delete)



from __future__ import annotations

import asyncio
import copy
import json
import logging
import os
import sys
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple

import jsonschema
from datayoga_core import blocks, utils
from datayoga_core.block import Block
from datayoga_core.context import Context
from datayoga_core.result import Result, Status
from datayoga_core.step import Step

logger = logging.getLogger("dy")


class ErrorHandling(Enum):
    ABORT = "abort"
    IGNORE = "ignore"


class Job():
    """
    Job

    Attributes:
        steps List[Block]: List of steps
    """

    def __init__(self, steps: Optional[List[Step]] = None, input: Optional[Block] = None,
                 error_handling: Optional[ErrorHandling] = None):
        """
        Constructs a job and its blocks

        Args:
            steps (List[Dict[str, Any]]): Job steps
            input (Optional[Block]): Block to be used as a producer
            error_handling (Optional[ErrorHandling]): error handling strategy
        """
        self.input = input
        self.steps = steps
        self.error_handling = error_handling if error_handling else ErrorHandling.IGNORE
        self.initialized = False

    def init(self, context: Optional[Context] = None):
        # open any connections or setup needed
        self.context = context
        self.root = None
        for step in self.steps:
            step.init(context)

            # create the step sequence
            if self.root is None:
                self.root = step
                last_step = self.root
            else:
                last_step = last_step.append(step)

        if self.input:
            self.input.init(context)

        self.root.add_done_callback(self.handle_results)
        self.initialized = True

    def transform(self, data: List[Dict[str, Any]]) -> Tuple[List[Dict[str, Any]], List[Result]]:
        """
        Transforms data

        Args:
            data (List[Dict[str, Any]]): Data

        Returns:
            Tuple[List[Dict[str, Any]], List[Result]]: Transformed data and results
        """
        if not self.initialized:
            logger.debug("job has not been initialized yet, initializing...")
            self.init()

        transformed_data = copy.deepcopy(data)
        results = []
        for step in self.steps:
            try:
                transformed_data, results = asyncio.run(step.block.run(transformed_data))
            except ConnectionError as e:
                # connection errors are thrown back to the caller to handle
                raise e
            except Exception as e:
                # other exceptions are rejected
                logger.error(f"Error while transforming data: {e}")
                utils.reject_records(data, f"{e}")
                transformed_data, results = utils.produce_data_and_results(data)
                break

        return transformed_data, results

    async def run(self):
        for record in self.input.produce():
            logger.debug(f"Retrieved record:\n\t{record}")
            await self.root.process([record])

        await self.shutdown()

    async def shutdown(self):
        # wait for in-flight records to finish
        await self.root.join()

        # graceful shutdown
        await self.root.stop()

    def handle_results(self, msg_ids: List[str], results: List[Result]):
        if any(x.status == Status.REJECTED for x in results) and self.error_handling == ErrorHandling.ABORT.value:
            logger.critical("Aborting due to rejected record(s)")
            sys.exit(1)

        self.input.ack(msg_ids)

    @staticmethod
    def validate(source: Dict[str, Any], whitelisted_blocks: Optional[List[str]] = None):
        jsonschema.validate(instance=source, schema=Job.get_json_schema(whitelisted_blocks))

    @staticmethod
    def compile(source: Dict[str, Any], whitelisted_blocks: Optional[List[str]] = None) -> Job:
        Job.validate(source, whitelisted_blocks=whitelisted_blocks)
        steps: List[Step] = []
        # parse the steps
        for step_definition in source.get("steps"):
            block_type = step_definition.get("uses")
            block: Block = Block.create(block_type, step_definition.get("with"))
            step: Step = Step(step_definition.get("id", block_type), block)
            steps.append(step)

        # parse the input
        input = None
        if source.get("input") is not None:
            input_definition = source.get("input")
            input = Block.create(input_definition.get("uses"), input_definition.get("with"))

        return Job(steps, input, source.get("error_handling"))

    @staticmethod
    def get_json_schema(whitelisted_blocks: Optional[List[str]] = None) -> Dict[str, Any]:
        # compiles a complete json schema of the job and all possible blocks
        block_schemas = []
        # we traverse the json schemas directly instead of 'walk_packages'
        # to avoid importing all of the block classes
        schema_paths = Path(os.path.join(utils.get_bundled_dir(), "blocks") if utils.is_bundled() else os.path.dirname(
            os.path.realpath(blocks.__file__))).rglob("**/block.schema.json")

        for schema_path in schema_paths:
            block_type = os.path.relpath(
                os.path.dirname(schema_path),
                os.path.dirname(os.path.realpath(blocks.__file__))
            )
            block_type = block_type.replace(os.path.sep, ".")

            if not (whitelisted_blocks is not None and block_type not in whitelisted_blocks):
                # load schema file
                schema = utils.read_json(f"{schema_path}")
                # append to the array of oneOf for the full schema
                block_schemas.append({
                    "type": "object",
                    "properties": {
                        "uses": {
                            "description": "Block type",
                            "type": "string",
                            "const": block_type
                        },
                        "with": schema,
                    },
                    "additionalProperties": False,
                    "required": ["uses"],
                })

        job_schema = utils.read_json(
            os.path.join(
                utils.get_bundled_dir() if utils.is_bundled() else os.path.dirname(os.path.realpath(__file__)),
                "resources", "schemas", "job.schema.json"))
        job_schema["definitions"]["block"] = {
            "type": "object",
            "oneOf": block_schemas
        }

        return job_schema



import json
import logging
from typing import Any, Dict, List, Tuple

from datayoga_core import expression, utils
from datayoga_core.block import Block as DyBlock
from datayoga_core.block import Result
from datayoga_core.context import Context

logger = logging.getLogger("dy")


class Block(DyBlock):
    def init(self, context: Context = None):
        logger.debug(f"Initializing {self.get_block_name()}")
        expression_prop = json.dumps(
            self.properties["expression"]) if isinstance(
            self.properties["expression"],
            dict) else self.properties["expression"]
        # the expression is now a string (jmespath expressions are not valid JSON)
        expression_prop = expression_prop.strip()
        if not (expression_prop.startswith("{") and expression_prop.endswith("}")):
            raise ValueError("map expression must be in a json-like format enclosed in { }")
        self.expression = expression.compile(self.properties["language"], expression_prop)

    async def run(self, data: List[Dict[str, Any]]) -> Tuple[List[Dict[str, Any]], List[Result]]:
        logger.debug(f"Running {self.get_block_name()}")
        mapped_rows = self.expression.search_bulk(data)
        # we always add the internal fields back, in case they were dropped in the map operation
        for mapped_row,original_row in zip(mapped_rows,data):
            original_msg_id= original_row.get(Block.MSG_ID_FIELD)
            original_opcode= original_row.get(Block.OPCODE_FIELD)
            if original_msg_id is not None:
                mapped_row[Block.MSG_ID_FIELD] = original_msg_id
            if original_opcode is not None:
                mapped_row[Block.OPCODE_FIELD] = original_opcode

        return utils.all_success(mapped_rows)



import json
import logging
import sqlite3
from abc import abstractmethod
from collections.abc import MutableMapping
from enum import Enum, unique
from typing import Any, Dict, List, Union

import jmespath
from datayoga_core.jmespath_custom_functions import JmespathCustomFunctions

logger = logging.getLogger("dy")


@unique
class Language(Enum):
    JMESPATH = "jmespath"
    SQL = "sql"


def flatten_data(data: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    # flattened structure
    data_inner = data if isinstance(data, list) else [data]
    data_inner = [flatten(row, sep=".") for row in data_inner]
    return data_inner


def flatten(d, parent_key="", sep="_"):
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, MutableMapping):
            items.extend(flatten(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


class Expression():
    @abstractmethod
    def compile(self, expression: str):
        """Compiles an expression

        Args:
            expression (str): expression
        """
        raise NotImplementedError

    @abstractmethod
    def search(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Executes the expression on a single data entry

        Args:
            data (List[Dict[str, Any]]): Data

        Returns:
            List[Dict[str, Any]]: Transformed data
        """
        raise NotImplementedError

    @abstractmethod
    def search_bulk(self, data: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """Executes the expression in bulk on a list of data entries

        Args:
            data (List[Dict[str, Any]]): Data

        Returns:
            List[Dict[str, Any]]: Transformed data
        """
        raise NotImplementedError

    def filter(self, data: List[Dict[str, Any]], tombstone: bool = False) -> List[Union[Dict[str, Any], None]]:
        """Tests a where clause for an SQL statement

        Args:
            data (List[Dict[str, Any]]): Data
            tombestone: if True, returns None for filtered values

        Returns:
            List[Dict[str, Any]]: Filtered data
        """
        # filter is essentially using a single field returning 0 or 1 for the condition
        if tombstone:
            return [row[0] if row[1] else None for row in zip(data,self.search_bulk(data))]
        else:
            return [row[0] for row in zip(data,self.search_bulk(data)) if row[1]]

class SQLExpression(Expression):
    def compile(self, expression: str):
        # we turn off `check_same_thread` to gain performance benefit by reusing the same connection object
        # safe to use since we are only creating in memory structures
        self.conn = sqlite3.connect(":memory:", check_same_thread=False)
        # we support both single field expressions and multiple fields
        self._is_single_field = True
        try:
            self._fields = json.loads(expression)
            self._is_single_field = False
        except json.JSONDecodeError:
            # this is not a json, treat as a simple expression
            self._fields = {"expr": expression}

    def _get_cte(self, data: List[Any]) -> str:
        # builds a CTE expression for fetching in memory data
        column_names = data[0].keys()
        columns_clause = ','.join(f"`{col}`" for col in column_names)

        # values in the form of (?,?), (?,?)
        values_clause_row = f"({','.join('?'*len(column_names))})"
        values_clause = ','.join([values_clause_row]*len(data))

        # use a CTE to create the in memory data structure
        return f"with data({columns_clause}) as (values {values_clause})"

    def search_bulk(self, data: List[Dict[str, Any]]) -> Any:
        results = self.exec_sql(data, self._fields)
        if self._is_single_field:
            # treat as a simple expression
            return [x.get("expr") for x in results]
        else:
            return results

    def search(self, data: Dict[str, Any]) -> Any:
        return self.search_bulk([data])[0]

    def exec_sql(self, data: List[Dict[str, Any]], expressions: Dict[str, str]) -> List[Dict[str, Any]]:
        """Executes an SQL statement

        Args:
            data (List[Dict[str, Any]]): Data
            expressions: Dict[str, str]: Expressions

        Returns:
            List[Dict[str, Any]]: Query result
        """
        # use a CTE to create the in memory data structure
        data_inner = flatten_data(data)
        cte_clause = self._get_cte(data_inner)

        column_names = data_inner[0].keys()

        # fetch the CTE and bind the variables
        data_values = [row.get(colname) for row in data_inner for colname in column_names]

        # expressions clause
        expressions_clause = ", ".join([f"{expression} as `{column_name}`" for column_name,expression in expressions.items()])
        self.conn.row_factory = sqlite3.Row
        statement = f"{cte_clause} select {expressions_clause} from data"
        logger.debug(statement)
        cursor = self.conn.execute(statement, data_values)
        return [dict(x) for x in cursor.fetchall()]


class JMESPathExpression(Expression):
    # register custom functions
    options = jmespath.Options(custom_functions=JmespathCustomFunctions())

    def compile(self, expression: str):
        self.expression = jmespath.compile(expression)

    def search(self, data: Dict[str, Any]) -> Any:
        return self.expression.search(data, options=self.options)

    def search_bulk(self, data: List[Dict[str, Any]]) -> Any:
        return [self.search(row) for row in data]


def compile(language: Language, expression: str) -> Expression:
    """Gets a compiled expression class based on the language

    Args:
        language (Language): Language
        expression (str): Expression

    Returns:
        Expression: Expression class
    """
    if language == Language.JMESPATH.value:
        expression_class = JMESPathExpression()
    elif language == Language.SQL.value:
        expression_class = SQLExpression()
    else:
        raise ValueError(f"unknown expression language {language}")

    expression_class.compile(expression)
    return expression_class



import logging
from typing import Any, Dict, List, Tuple, Union

from datayoga_core import utils
from datayoga_core.opcode import OPCODES, OpCode

logger = logging.getLogger("dy")


def group_records_by_opcode(records: List[Dict[str, Any]],
                            opcode_field: str,
                            keys: List[Union[Dict[str, str], str]]) -> Tuple[List[Dict[str, Any]],
                                                                             List[Dict[str, Any]],
                                                                             List[Dict[str, Any]]]:
    # group records by their opcode, reject records with unsupported opcode or with missing keys
    records_to_update: List[Dict[str, Any]] = []
    records_to_delete: List[Dict[str, Any]] = []
    records_to_insert: List[Dict[str, Any]] = []

    for record in records:
        opcode = record.get(opcode_field)
        if opcode is None:
            utils.reject_record(f"{opcode_field} opcode field does not exist", record)
        elif opcode in OPCODES:
            for item in keys:
                source = next(iter(item.values())) if isinstance(item, dict) else item
                if source not in record:
                    utils.reject_record(f"{source} key does not exist", record)
                    break
        else:
            utils.reject_record(f"{opcode} - unsupported opcode", record)

        if not utils.is_rejected(record):
            if opcode == OpCode.CREATE.value:
                records_to_insert.append(record)
            elif opcode == OpCode.UPDATE.value:
                records_to_update.append(record)
            elif opcode == OpCode.DELETE.value:
                records_to_delete.append(record)

    return records_to_insert, records_to_update, records_to_delete


def get_column_mapping(mapping: List[Union[Dict[str, str], str]]) -> List[Dict[str, str]]:
    return [{"column": next(iter(item.keys())), "key": next(iter(item.values()))} if isinstance(item, dict)
            else {"column": item, "key": item} for item in mapping] if mapping else []


def map_record(record: Dict[str, Any],
               keys: List[Union[Dict[str, str], str]],
               mapping: List[Union[Dict[str, str], str]] = []):
    # map the record based on the mapping definitions
    # add nulls for missing mapping fields
    mapped_record = {}
    for item in keys + mapping:
        source = next(iter(item.values())) if isinstance(item, dict) else item

        # columns with spaces will be later used with underscores in the bind variables
        target = (next(iter(item.keys())) if isinstance(item, dict) else item).replace(" ", "_")
        mapped_record[target] = None if source not in record else record[source]

    return mapped_record



from __future__ import annotations

import importlib
import logging
import os
import sys
from abc import ABCMeta, abstractmethod
from os import path
from typing import Any, Dict, List, Optional, Tuple

from jsonschema import validate

from datayoga_core import utils
from datayoga_core.context import Context
from datayoga_core.result import Result

logger = logging.getLogger("dy")


class Block(metaclass=ABCMeta):
    INTERNAL_FIELD_PREFIX = "__$$"
    MSG_ID_FIELD = f"{INTERNAL_FIELD_PREFIX}msg_id"
    RESULT_FIELD = f"{INTERNAL_FIELD_PREFIX}result"
    OPCODE_FIELD = f"{INTERNAL_FIELD_PREFIX}opcode"
    """
    Block

    Attributes:
        properties Dict[str, Any]: Block properties
    """

    def __init__(self, properties: Dict[str, Any] = {}):
        """
        Constructs a block

        Args:
            properties (Dict[str, Any]): Block [properties]
        """
        self.properties = properties
        self.validate()

    def validate(self):
        """
        Validates block against its JSON Schema
        """
        logger.debug(f"validating {self.properties}")
        validate(instance=self.properties, schema=self.get_json_schema())

    def get_json_schema(self) -> Dict[str, Any]:
        """
        Returns the JSON Schema for this block

        Returns:
            Dict[str, Any]: JSON Schema
        """
        json_schema_file = path.join(
            utils.get_bundled_dir(),
            os.path.relpath(
                os.path.dirname(sys.modules[self.__module__].__file__),
                start=os.path.dirname(__file__)),
            "block.schema.json") if utils.is_bundled() else path.join(
            os.path.dirname(os.path.realpath(sys.modules[self.__module__].__file__)),
            "block.schema.json")
        logger.debug(f"loading schema from {json_schema_file}")
        return utils.read_json(json_schema_file)

    @abstractmethod
    def init(self, context: Optional[Context] = None):
        """
        Initializes block

        Args:
            context (Context, optional): Context. Defaults to None.
        """
        raise NotImplementedError

    async def run(self, data: List[Dict[str, Any]]) -> Tuple[List[Dict[str, Any]], List[Result]]:
        """ Transforms data

        Args:
            data (List[Dict[str, Any]]): Data

        Returns:
            Tuple[List[Dict[str, Any]], List[Result]]: Transformed data and results
        """
        pass

    def get_block_name(self):
        return os.path.basename(os.path.dirname(sys.modules[self.__module__].__file__))

    @staticmethod
    def create(block_name: str, properties: Dict[str, Any]) -> Block:
        module_name = f"datayoga_core.blocks.{block_name}.block"
        module = importlib.import_module(module_name)
        block: Block = getattr(module, "Block")(properties)
        return block



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['datayoga_core',
 'datayoga_core.blocks',
 'datayoga_core.blocks.add_field',
 'datayoga_core.blocks.add_field.tests',
 'datayoga_core.blocks.cassandra',
 'datayoga_core.blocks.cassandra.write',
 'datayoga_core.blocks.files.read_csv',
 'datayoga_core.blocks.filter',
 'datayoga_core.blocks.filter.tests',
 'datayoga_core.blocks.map',
 'datayoga_core.blocks.map.tests',
 'datayoga_core.blocks.redis',
 'datayoga_core.blocks.redis.read_stream',
 'datayoga_core.blocks.redis.write',
 'datayoga_core.blocks.relational',
 'datayoga_core.blocks.relational.write',
 'datayoga_core.blocks.remove_field',
 'datayoga_core.blocks.remove_field.tests',
 'datayoga_core.blocks.rename_field',
 'datayoga_core.blocks.rename_field.tests',
 'datayoga_core.blocks.sequence',
 'datayoga_core.blocks.std',
 'datayoga_core.blocks.std.read',
 'datayoga_core.blocks.std.write']

package_data = \
{'': ['*'],
 'datayoga_core': ['resources/scaffold/*',
                   'resources/scaffold/data/*',
                   'resources/scaffold/jobs/sample/*',
                   'resources/schemas/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'certifi>=2022.12.7,<2023.0.0',
 'click>=8.0.3,<9.0.0',
 'cryptography>=38.0.4,<39.0.0',
 'jmespath>=1.0.0,<2.0.0',
 'jsonschema>=4.4.0,<5.0.0']

extras_require = \
{'cassandra': ['cassandra-driver>=3.25.0,<4.0.0'],
 'mssql': ['pymssql>=2.2.7,<3.0.0', 'SQLAlchemy>=1.4.44,<2.0.0'],
 'mysql': ['PyMySQL>=1.0.2,<2.0.0', 'SQLAlchemy>=1.4.44,<2.0.0'],
 'pg': ['psycopg2-binary>=2.9.5,<3.0.0', 'SQLAlchemy>=1.4.44,<2.0.0'],
 'redis': ['redis>=4.3.5,<5.0.0'],
 'test': ['mock>=4.0.3,<5.0.0',
          'pytest>=7.1.2,<8.0.0',
          'pytest-asyncio>=0.20.2,<0.21.0',
          'pytest-describe>=2.0.1,<3.0.0',
          'pytest-mock>=3.7.0,<4.0.0',
          'pytest-timeout>=2.1.0,<3.0.0',
          'requests-mock>=1.9.3,<2.0.0',
          'testcontainers>=3.7.0,<4.0.0',
          'cassandra-driver>=3.25.0,<4.0.0',
          'psycopg2-binary>=2.9.5,<3.0.0',
          'pymssql>=2.2.7,<3.0.0',
          'PyMySQL>=1.0.2,<2.0.0',
          'redis>=4.3.5,<5.0.0',
          'SQLAlchemy>=1.4.44,<2.0.0']}

setup_kwargs = {
    'name': 'datayoga-core',
    'version': '1.41.0',
    'description': 'DataYoga for Python',
    'long_description': '# DataYoga Core\n\n## Introduction\n\n`datayoga-core` is the transformation engine used in `DataYoga`, a framework for building and generating data pipelines.\n\n## Installation\n\n```bash\npip install datayoga-core\n```\n\n## Quick Start\n\nThis demonstrates how to transform data using a DataYoga job.\n\n### Create a Job\n\nUse this `example.yaml`:\n\n```yaml\n- steps:\n    - uses: add_field\n      with:\n        fields:\n          - field: full_name\n            language: jmespath\n            expression: concat([fname, \' \' , lname])\n          - field: country\n            language: sql\n            expression: country_code || \' - \' || UPPER(country_name)\n    - uses: rename_field\n      with:\n        fields:\n          - from_field: fname\n            to_field: first_name\n          - from_field: lname\n            to_field: last_name\n    - uses: remove_field\n      with:\n        fields:\n          - field: credit_card\n          - field: country_name\n          - field: country_code\n    - uses: map\n      with:\n        expression:\n          {\n            first_name: first_name,\n            last_name: last_name,\n            greeting: "\'Hello \' || CASE WHEN gender = \'F\' THEN \'Ms.\' WHEN gender = \'M\' THEN \'Mr.\' ELSE \'N/A\' END || \' \' || full_name",\n            country: country,\n            full_name: full_name\n          }\n        language: sql\n```\n\n### Transform Data Using `datayoga-core`\n\nUse this code snippet to transform a data record using the job defined [above](#create-a-job):\n\n```python\nimport datayoga_core as dy\nfrom datayoga_core.job import Job\nfrom datayoga_core.utils import read_yaml\n\njob_settings = read_yaml("example.yaml")\njob = dy.compile(job_settings)\n\nassert job.transform({"fname": "jane", "lname": "smith", "country_code": 1, "country_name": "usa", "credit_card": "1234-5678-0000-9999", "gender": "F"}) == {"first_name": "jane", "last_name": "smith", "country": "1 - USA", "full_name": "jane smith", "greeting": "Hello Ms. jane smith"}\n```\n\nAs can be seen, the record has been transformed based on the job:\n\n- `fname` field renamed to `first_name`.\n- `lname` field renamed to `last_name`.\n- `country` field added based on an SQL expression.\n- `full_name` field added based on a [JMESPath](https://jmespath.org/) expression.\n- `greeting` field added based on an SQL expression.\n\n### Examples\n\n- Add a new field `country` out of an SQL expression that concatenates `country_code` and `country_name` fields after upper case the later:\n\n  ```yaml\n  uses: add_field\n  with:\n    field: country\n    language: sql\n    expression: country_code || \' - \' || UPPER(country_name)\n  ```\n\n- Rename `fname` field to `first_name` and `lname` field to `last_name`:\n\n  ```yaml\n  uses: rename_field\n  with:\n    fields:\n      - from_field: fname\n        to_field: first_name\n      - from_field: lname\n        to_field: last_name\n  ```\n\n- Remove `credit_card` field:\n\n  ```yaml\n  uses: remove_field\n  with:\n    field: credit_card\n  ```\n\nFor a full list of supported block types [see reference](https://datayoga-io.github.io/library).\n\n## Expression Language\n\nDataYoga supports both SQL and [JMESPath](https://jmespath.org/) expressions. JMESPath are especially useful to handle nested JSON data, while SQL is more suited to flat row-like structures.\n\nFor more information about custom functions and supported expression language syntax [see reference](https://datayoga-io.github.io/expressions).\n',
    'author': 'DataYoga',
    'author_email': 'admin@datayoga.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
